/******************************************************************************/
*
* Predictable Financial Crises
* by Greenwood, Hanson, Shleifer and Sorensen (2021)
*
* Code for replication
* Tables 1-11
*
* Date: 05-22-2021
*
/******************************************************************************/

clear all       
set more off    

* Set directory to folder with "Data.csv" and the xtsccfixedb.ado files
cd "/Users/jakobahm/Dropbox (CBS)/Work/Research/Predictable Financial Crises/ReplicationKit"


/******************************************************************************/
*
* Import data and define variables
*
/******************************************************************************/



* Import data
insheet using "Data.csv", clear

* Structure data
egen country_num = group(country)
tsset country_num year
format crisis_ind* %9.0f


* Define indicator variables for future crises, equity crashes, bank failures and panics
local indlist = "crisis_ind_bvx crisis_ind_st crisis_ind_rr bvx_bank_eq_decline bvx_bank_failures crisis_ind_bvx_panic crisis_ind_bvx_equity"
foreach ind of local indlist {
	replace `ind' = 100*`ind'
	
	gen f1_`ind' = f.`ind'
	gen f2_`ind' = f2.`ind'
	gen f3_`ind' = f3.`ind'
	gen f4_`ind' = f4.`ind'
	
	gen f1_cum_`ind' = f1_`ind'
	gen f2_cum_`ind' = max(f2_`ind',f1_`ind') if f1_`ind' != . & f2_`ind' != .
	gen f3_cum_`ind' = max(f3_`ind',f2_`ind',f1_`ind') if f1_`ind' != . & f2_`ind' != . & f3_`ind' != .
	gen f4_cum_`ind' = max(f4_`ind',f3_`ind',f2_`ind',f1_`ind') if f1_`ind' != . & f2_`ind' != . & f3_`ind' != . & f4_`ind' != .
	
}

/* Define cumulative 3-year economic contraction probability */
gen f1_contract_gdp_ind = 100*(f1.gdp_real_lg1 < -2) if f1.gdp_real_lg1 != .
gen f2_contract_gdp_ind = 100*(f2.gdp_real_lg1 < -2) if f2.gdp_real_lg1 != .
gen f3_contract_gdp_ind = 100*(f3.gdp_real_lg1 < -2) if f3.gdp_real_lg1 != .
gen f4_contract_gdp_ind = 100*(f4.gdp_real_lg1 < -2) if f4.gdp_real_lg1 != .

gen f1_cum_contract_gdp_ind = f1_contract_gdp_ind
gen f2_cum_contract_gdp_ind = max(f2_contract_gdp_ind,f1_contract_gdp_ind) if f1_contract_gdp_ind != . & f2_contract_gdp_ind != .
gen f3_cum_contract_gdp_ind = max(f3_contract_gdp_ind,f2_contract_gdp_ind,f1_contract_gdp_ind) if f1_contract_gdp_ind != . & f2_contract_gdp_ind != . & f3_contract_gdp_ind != .
gen f4_cum_contract_gdp_ind = max(f4_contract_gdp_ind,f3_contract_gdp_ind,f2_contract_gdp_ind,f1_contract_gdp_ind) if f1_contract_gdp_ind != . & f2_contract_gdp_ind != . & f3_contract_gdp_ind != . & f4_contract_gdp_ind != .

/* Define cumulative future real GDP growth*/
gen f1_cum_avg_gdp_real = f1.gdp_real_lg1
gen f2_cum_avg_gdp_real = (f1.gdp_real_lg1 + f2.gdp_real_lg1)/2
gen f3_cum_avg_gdp_real = (f1.gdp_real_lg1 + f2.gdp_real_lg1 + f3.gdp_real_lg1)/3
gen f4_cum_avg_gdp_real = (f1.gdp_real_lg1 + f2.gdp_real_lg1 + f3.gdp_real_lg1 + f4.gdp_real_lg1)/4

gen f1_cum_gdp_real = 1*f1_cum_avg_gdp_real
gen f2_cum_gdp_real = 2*f2_cum_avg_gdp_real
gen f3_cum_gdp_real = 3*f3_cum_avg_gdp_real
gen f4_cum_gdp_real = 4*f4_cum_avg_gdp_real

/* Define samples */
* "Business sample": All country-years prior to 2013 where we can measure 3 year business debt growth and real equity price growth, and where the Baron, Verner and Xiong (2021) crisis variable is available 1 year ahead.
* "Housing sample": All country-years prior to 2013 where we can measure 3 year household debt growth and real house price growth, and where the Baron, Verner and Xiong (2021) crisis variable is available 1 year ahead.
* "Private sample": All country-years prior to 2013 where we can measure 3 year growth in private real debt, and where the Baron, Verner and Xiong (2021) crisis variable is available 1 year ahead.
gen sample_bus = (f1_crisis_ind_bvx != .)*(debt_to_gdp_bus_d3 != .)*(equities_real_lg3 !=.)*(year <= 2012)
gen sample_hh = (f1_crisis_ind_bvx != .)*(debt_to_gdp_hh_d3 != .)*(housing_real_lg3 !=.)*(year <= 2012)
gen sample_private = (f1_crisis_ind_bvx != .)*(debt_private_real_lg3 != .)*(year <= 2012)


/* Define standardized variables for Table 2 */
egen std_debt_to_gdp_private_d3 = std(debt_to_gdp_private_d3) if sample_private == 1
egen std_debt_private_real_lg3 = std(debt_private_real_lg3) if sample_private == 1
egen std_debt_to_gdp_bus_d3 = std(debt_to_gdp_bus_d3) if sample_bus == 1
egen std_debt_to_gdp_hh_d3 = std(debt_to_gdp_hh_d3) if sample_hh == 1

/* Define standard (forward-looking): R-zone definition */
xtile debt_bus_quint = debt_to_gdp_bus_d3 if sample_bus == 1, nquantiles(5)	
xtile price_bus_terc = equities_real_lg3 if sample_bus == 1, nquantiles(3)	
xtile debt_hh_quint = debt_to_gdp_hh_d3 if sample_hh == 1, nquantiles(5)	
xtile price_hh_terc = housing_real_lg3 if sample_hh == 1, nquantiles(3)	
gen debt_bus_high = (debt_bus_quint == 5) if sample_bus == 1
gen price_bus_high = (price_bus_terc == 3) if sample_bus == 1
gen rzone_bus = debt_bus_high*price_bus_high
gen debt_hh_high = (debt_hh_quint == 5) if sample_hh == 1
gen price_hh_high = (price_hh_terc == 3) if sample_hh == 1
gen rzone_hh = debt_hh_high*price_hh_high
gen rzone_both = rzone_bus*rzone_hh
gen rzone_either = max(rzone_bus,rzone_hh)
gen rzone_either_mod = max(rzone_bus,rzone_hh) if rzone_bus != . & rzone_hh != .


/* Define R-zone using rolling (backward-looking) definition (Table 5.i)*/
gen debt_bus_quint_r = .
gen price_bus_terc_r = .
gen debt_hh_quint_r = .
gen price_hh_terc_r = .	

forvalues year = 1973/2015 {
	qui xtile debt_bus_quint_r_temp = debt_to_gdp_bus_d3 if sample_bus == 1 & year <= `year', nquantiles(5)	
	qui xtile price_bus_terc_r_temp = equities_real_lg3 if sample_bus == 1 & year <= `year', nquantiles(3)
	qui replace debt_bus_quint_r = debt_bus_quint_r_temp if year == `year'
	qui replace price_bus_terc_r = price_bus_terc_r_temp if year == `year'
	qui drop debt_bus_quint_r_temp price_bus_terc_r_temp
	
	qui xtile debt_hh_quint_r_temp = debt_to_gdp_hh_d3 if sample_hh == 1 & year <= `year', nquantiles(5)	
	qui xtile price_hh_terc_r_temp = housing_real_lg3 if sample_hh == 1 & year <= `year', nquantiles(3)
	qui replace debt_hh_quint_r = debt_hh_quint_r_temp if year == `year'
	qui replace price_hh_terc_r = price_hh_terc_r_temp if year == `year'
	qui drop debt_hh_quint_r_temp price_hh_terc_r_temp
}

gen debt_bus_high_r = (debt_bus_quint_r == 5) if sample_bus == 1 & debt_bus_quint_r != .
gen price_bus_high_r = (price_bus_terc_r == 3) if sample_bus == 1 & price_bus_terc_r != .
gen rzone_bus_r = debt_bus_high_r*price_bus_high_r
gen debt_hh_high_r = (debt_hh_quint_r == 5) if sample_hh == 1 & debt_hh_quint_r != .
gen price_hh_high_r = (price_hh_terc_r == 3) if sample_hh == 1 & price_hh_terc_r != .
gen rzone_hh_r = debt_hh_high_r*price_hh_high_r
gen rzone_both_r = rzone_bus_r*rzone_hh_r
gen rzone_either_r = max(rzone_bus_r,rzone_hh_r)

/* Define the R-zone using the "leave-out" definition: leave out 3 years prior and 4 years after (Table 5.ii) */
gen debt_bus_quint_j = .
gen price_bus_terc_j = .
gen debt_hh_quint_j = .
gen price_hh_terc_j = .	

forvalues year = 1950/2015 {
	qui xtile debt_bus_quint_j_temp = debt_to_gdp_bus_d3 if sample_bus == 1 & (year < (`year'- 3) | year > (`year' + 7)), nquantiles(5)	
	qui xtile price_bus_terc_j_temp = equities_real_lg3 if sample_bus == 1 & (year < (`year'- 3) | year > (`year' + 7)), nquantiles(3)
	gsort sample_bus -debt_to_gdp_bus_d3 -year
	replace debt_bus_quint_j_temp = debt_bus_quint_j_temp[_n-1] if sample_bus == 1 & debt_bus_quint_j_temp == . & debt_bus_quint_j_temp[_n-1] != .
	gsort sample_bus -equities_real_lg3 -year
	replace price_bus_terc_j_temp = price_bus_terc_j_temp[_n-1] if sample_bus == 1 & price_bus_terc_j_temp == . & price_bus_terc_j_temp[_n-1] != .	
	qui replace debt_bus_quint_j = debt_bus_quint_j_temp if year == `year'
	qui replace price_bus_terc_j = price_bus_terc_j_temp if year == `year'
	qui drop debt_bus_quint_j_temp price_bus_terc_j_temp
	
	qui xtile debt_hh_quint_j_temp = debt_to_gdp_hh_d3 if sample_hh == 1 & (year < (`year'- 3) | year > (`year' + 7)), nquantiles(5)	
	qui xtile price_hh_terc_j_temp = housing_real_lg3 if sample_hh == 1 & (year < (`year'- 3) | year > (`year' + 7)), nquantiles(3)
	gsort sample_hh -debt_to_gdp_hh_d3 -year
	replace debt_hh_quint_j_temp = debt_hh_quint_j_temp[_n-1] if sample_hh == 1 & debt_hh_quint_j_temp == . & debt_hh_quint_j_temp[_n-1] != .
	gsort sample_hh -housing_real_lg3 -year
	replace price_hh_terc_j_temp = price_hh_terc_j_temp[_n-1] if sample_hh == 1 & price_hh_terc_j_temp == . & price_hh_terc_j_temp[_n-1] != .	
	qui replace debt_hh_quint_j = debt_hh_quint_j_temp if year == `year'
	qui replace price_hh_terc_j = price_hh_terc_j_temp if year == `year'
	qui drop debt_hh_quint_j_temp price_hh_terc_j_temp
}

sum debt_to_gdp_bus_d3 if sample_bus == 1 & debt_bus_quint_j  != .
replace debt_bus_quint_j  = 1 if sample_bus == 1 & debt_bus_quint_j  == . & debt_to_gdp_bus_d3 < r(min)
replace debt_bus_quint_j  = 5 if sample_bus == 1 & debt_bus_quint_j  == . & debt_to_gdp_bus_d3 > r(max)
sum equities_real_lg3 if sample_bus == 1 & price_bus_terc_j  != .
replace price_bus_terc_j  = 1 if sample_bus == 1 & price_bus_terc_j  == . & equities_real_lg3 < r(min)
replace price_bus_terc_j  = 3 if sample_bus == 1 & price_bus_terc_j  == . & equities_real_lg3 > r(max)

sum debt_to_gdp_hh_d3 if sample_hh == 1 & debt_hh_quint_j  != .
replace debt_hh_quint_j  = 1 if sample_hh == 1 & debt_hh_quint_j  == . & debt_to_gdp_hh_d3 < r(min)
replace debt_hh_quint_j  = 5 if sample_hh == 1 & debt_hh_quint_j  == . & debt_to_gdp_hh_d3 > r(max)
replace debt_hh_quint_j  = 5 if sample_hh == 1 & debt_hh_quint_j  == . 
sum housing_real_lg3 if sample_hh == 1 & price_hh_terc_j  != .
replace price_hh_terc_j  = 1 if sample_hh == 1 & price_hh_terc_j  == . & housing_real_lg3 < r(min)
replace price_hh_terc_j  = 3 if sample_hh == 1 & price_hh_terc_j  == . & housing_real_lg3 > r(max)

gen debt_bus_high_j = (debt_bus_quint_j == 5) if sample_bus == 1 & debt_bus_quint_j != .
gen price_bus_high_j = (price_bus_terc_j == 3) if sample_bus == 1 & price_bus_terc_j != .
gen rzone_bus_j = debt_bus_high_j*price_bus_high_j
gen debt_hh_high_j = (debt_hh_quint_j == 5) if sample_hh == 1 & debt_hh_quint_j != .
gen price_hh_high_j = (price_hh_terc_j == 3) if sample_hh == 1 & price_hh_terc_j != .
gen rzone_hh_j = debt_hh_high_j*price_hh_high_j
gen rzone_both_j = rzone_bus_j*rzone_hh_j
gen rzone_either_j = max(rzone_bus_j,rzone_hh_j)


/* Define R-zone based on pre-2000 data (Table 5.iii) */
xtile debt_bus_quint_pre = debt_to_gdp_bus_d3 if sample_bus == 1 & year <= 1996, nquantiles(5)	
xtile price_bus_terc_pre = equities_real_lg3 if sample_bus == 1 & year <= 1996, nquantiles(3)	
xtile debt_hh_quint_pre = debt_to_gdp_hh_d3 if sample_hh == 1 & year <= 1996, nquantiles(5)	
xtile price_hh_terc_pre = housing_real_lg3 if sample_hh == 1 & year <= 1996, nquantiles(3)	
gen debt_bus_high_pre = (debt_bus_quint_pre == 5) if sample_bus == 1  & year <= 1996
gen price_bus_high_pre = (price_bus_terc_pre == 3) if sample_bus == 1  & year <= 1996
gen rzone_bus_pre = debt_bus_high_pre*price_bus_high_pre
gen debt_hh_high_pre = (debt_hh_quint_pre == 5) if sample_hh == 1 & year <= 1996
gen price_hh_high_pre = (price_hh_terc_pre == 3) if sample_hh == 1 & year <= 1996
gen rzone_hh_pre = debt_hh_high_pre*price_hh_high_pre
gen rzone_both_pre = rzone_bus_pre*rzone_hh_pre
gen rzone_either_pre = max(rzone_bus_pre,rzone_hh_pre)


/******************************************************************************/
*
* Table 1
* Summary Statistics
*
/******************************************************************************/


	
log using table1.log, replace

sum f1_crisis_ind_bvx f1_crisis_ind_st f1_crisis_ind_rr if (sample_hh == 1 | sample_bus == 1)
sum f1_bvx_bank_eq_decline f1_bvx_bank_failures f1_crisis_ind_bvx_panic if (sample_hh == 1 | sample_bus == 1)
sum f1_cum_gdp_real if (sample_hh == 1 | sample_bus == 1)
sum debt_to_gdp_bus_d3 debt_to_gdp_hh_d3 debt_private_real_lg3 if (sample_hh == 1 | sample_bus == 1)

centile debt_to_gdp_bus_d3 if sample_bus == 1, centile(20 40 60 80)
centile debt_to_gdp_hh_d3 if sample_hh == 1, centile(20 40 60 80)
centile debt_private_real_lg3 if sample_private == 1, centile(20 40 60 80)

sum equities_real_lg3 housing_real_lg3 if (sample_hh == 1 | sample_bus == 1)

centile equities_real_lg3 if sample_bus == 1, centile(33.33 66.67)
centile housing_real_lg3 if sample_hh == 1, centile(33.33 66.67)

log close


/******************************************************************************/
*
* Table 2
* Linear Regressions
*
/******************************************************************************/
log using table2.log, replace
xtsccfixedb f1_cum_crisis_ind_bvx std_debt_to_gdp_private_d3 if sample_private == 1, fe lag(0)	
xtsccfixedb f1_cum_crisis_ind_bvx std_debt_to_gdp_bus_d3 if sample_bus == 1, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx std_debt_to_gdp_hh_d3 if sample_hh == 1, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx std_debt_private_real_lg3 if sample_private == 1, fe lag(0)

xtsccfixedb f2_cum_crisis_ind_bvx std_debt_to_gdp_private_d3 if sample_private == 1, fe lag(3)	
xtsccfixedb f2_cum_crisis_ind_bvx std_debt_to_gdp_bus_d3 if sample_bus == 1, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx std_debt_to_gdp_hh_d3 if sample_hh == 1, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx std_debt_private_real_lg3 if sample_private == 1, fe lag(3)

xtsccfixedb f3_cum_crisis_ind_bvx std_debt_to_gdp_private_d3 if sample_private == 1, fe lag(5)	
xtsccfixedb f3_cum_crisis_ind_bvx std_debt_to_gdp_bus_d3 if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx std_debt_to_gdp_hh_d3 if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx std_debt_private_real_lg3 if sample_private == 1, fe lag(5)
log close


/******************************************************************************/
*
* Table 3
* Crisis Probabilities by Price and Debt Growth Quantiles
*
/******************************************************************************/

* Define indicator variables:
forvalues p = 1/3 {
	forvalues d = 1/5 {
		gen bus_bin_`p'`d' = (price_bus_terc == `p')*(debt_bus_quint == `d')
		gen hh_bin_`p'`d'   = (price_hh_terc == `p')*(debt_hh_quint == `d')
	}
}

** Table 3a: Business Sector
log using table3a.log, replace

* Distribution of observations by growth in business debt and equity prices
sum bus_bin_* if sample_bus == 1

* Crisis probabilities by growth in business debt and equity prices
reg f1_cum_crisis_ind_bvx bus_bin_* if sample_bus == 1, noconstant
reg f2_cum_crisis_ind_bvx bus_bin_* if sample_bus == 1, noconstant
reg f3_cum_crisis_ind_bvx bus_bin_* if sample_bus == 1, noconstant
reg f4_cum_crisis_ind_bvx bus_bin_* if sample_bus == 1, noconstant

* Compare Crisis probabilities to median group
drop bus_bin_23
xtsccfixedb f1_cum_crisis_ind_bvx bus_bin_* if sample_bus == 1, lag(0)
xtsccfixedb f2_cum_crisis_ind_bvx bus_bin_* if sample_bus == 1, lag(3)
xtsccfixedb f3_cum_crisis_ind_bvx bus_bin_* if sample_bus == 1, lag(5)
xtsccfixedb f4_cum_crisis_ind_bvx bus_bin_* if sample_bus == 1, lag(6)

log close

** Table 3b: Household Sector
log using table3b.log, replace

* Distribution of observations by growth in household debt and house prices
sum hh_bin_* if sample_hh == 1

* Crisis probabilities by growth in household debt and house prices
reg f1_cum_crisis_ind_bvx hh_bin_* if sample_hh == 1, noconstant
reg f2_cum_crisis_ind_bvx hh_bin_* if sample_hh == 1, noconstant
reg f3_cum_crisis_ind_bvx hh_bin_* if sample_hh == 1, noconstant
reg f4_cum_crisis_ind_bvx hh_bin_* if sample_hh == 1, noconstant

* Compare Crisis probabilities to median group
drop hh_bin_23
xtsccfixedb f1_cum_crisis_ind_bvx hh_bin_* if sample_hh == 1, lag(0)
xtsccfixedb f2_cum_crisis_ind_bvx hh_bin_* if sample_hh == 1, lag(3)
xtsccfixedb f3_cum_crisis_ind_bvx hh_bin_* if sample_hh == 1, lag(5)
xtsccfixedb f4_cum_crisis_ind_bvx hh_bin_* if sample_hh == 1, lag(6)	

log close


drop bus_bin_* hh_bin_*


/******************************************************************************/
*
* Table 4
* Crisis Prediction with Debt Growth and Asset Appreciation by Sector
*
/******************************************************************************/


** Table 4: Crisis Prediction with Debt Growth and Real Asset Appreciation by Sector

* Panel A: Predicting crises with business debt growth and real equity price appreciation
log using table4a.log, replace

* Crisis within 1 year
xtsccfixedb f1_cum_crisis_ind_bvx debt_bus_high if sample_bus == 1, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx price_bus_high if sample_bus == 1, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx rzone_bus if sample_bus == 1, fe lag(0)

* Crisis within 2 years
xtsccfixedb f2_cum_crisis_ind_bvx debt_bus_high if sample_bus == 1, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx price_bus_high if sample_bus == 1, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx rzone_bus if sample_bus == 1, fe lag(3)

* Crisis within 3 years
xtsccfixedb f3_cum_crisis_ind_bvx debt_bus_high if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx price_bus_high if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus if sample_bus == 1, fe lag(5)

* Crisis within 4 years
xtsccfixedb f4_cum_crisis_ind_bvx debt_bus_high if sample_bus == 1, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx price_bus_high if sample_bus == 1, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx rzone_bus if sample_bus == 1, fe lag(6)

log close

* Panel B: Predicting crises with household debt growth and real house price appreciation
log using table4b.log, replace

* Crisis within 1 year
xtsccfixedb f1_cum_crisis_ind_bvx debt_hh_high if sample_hh == 1, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx price_hh_high if sample_hh == 1, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx rzone_hh if sample_hh == 1, fe lag(0)

* Crisis within 2 years
xtsccfixedb f2_cum_crisis_ind_bvx debt_hh_high if sample_hh == 1, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx price_hh_high if sample_hh == 1, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx rzone_hh if sample_hh == 1, fe lag(3)

* Crisis within 3 years
xtsccfixedb f3_cum_crisis_ind_bvx debt_hh_high if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx price_hh_high if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_hh if sample_hh == 1, fe lag(5)

* Crisis within 4 years
xtsccfixedb f4_cum_crisis_ind_bvx debt_hh_high if sample_hh == 1, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx price_hh_high if sample_hh == 1, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx rzone_hh if sample_hh == 1, fe lag(6)	

log close


/******************************************************************************/
*
* Table 5
* Crisis Prediction with Debt Growth and Asset Appreciation by Sector
*
/******************************************************************************/


** Panel A: Business Sector Robustness Table
log using table5a.log, replace	

* Business specification #1
xtsccfixedb f3_cum_crisis_ind_bvx debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_bus_high_r price_bus_high_r rzone_bus_r if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_bus_high_j price_bus_high_j rzone_bus_j if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_bus_high price_bus_high rzone_bus if sample_bus == 1 & year <= 1996, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_bus_high_pre price_bus_high_pre rzone_bus_pre if sample_bus == 1 & year <= 1996, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_st debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_rr debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_bvx_bank_eq_decline debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_bvx_bank_failures debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx_panic debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx_equity debt_bus_high price_bus_high rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_bus_high price_bus_high rzone_bus if sample_bus == 1 & developed == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_bus_high price_bus_high rzone_bus if sample_bus == 1 & developed == 0, fe lag(5)	

* Business specification #2
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus_r if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus_j if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus if sample_bus == 1 & year <= 1996, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus_pre if sample_bus == 1 & year <= 1996, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_st rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_rr rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_bvx_bank_eq_decline rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_bvx_bank_failures rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx_panic rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx_equity rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus if sample_bus == 1 & developed == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus if sample_bus == 1 & developed == 0, fe lag(5)	

log close

** Panel B: Household Sector Robustness Table
log using table5b.log, replace			
	
** Household specification #1**
xtsccfixedb f3_cum_crisis_ind_bvx debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_hh_high_r price_hh_high_r rzone_hh_r if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_hh_high_j price_hh_high_j rzone_hh_j if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_hh_high price_hh_high rzone_hh if sample_hh == 1 & year <= 1996, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_hh_high_pre price_hh_high_pre rzone_hh_pre if sample_hh == 1 & year <= 1996, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_st debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_rr debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_bvx_bank_eq_decline debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_bvx_bank_failures debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx_panic debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx_equity debt_hh_high price_hh_high rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_hh_high price_hh_high rzone_hh if sample_hh == 1 & developed == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx debt_hh_high price_hh_high rzone_hh if sample_hh == 1 & developed == 0, fe lag(5)

** Household specification #2**
xtsccfixedb f3_cum_crisis_ind_bvx rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_hh_r if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_hh_j if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_hh if sample_hh == 1 & year <= 1996, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_hh_pre if sample_hh == 1 & year <= 1996, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_st rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_rr rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_bvx_bank_eq_decline rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_bvx_bank_failures rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx_panic rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx_equity rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_hh if sample_hh == 1 & developed == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_hh if sample_hh == 1 & developed == 0, fe lag(5)
	
log close



/******************************************************************************/
*
* Table 6
* Crisis Prediction with data from both households and businesses
*
/******************************************************************************/
	
log using table6.log, replace	

* Crisis within 1 year
xtsccfixedb f1_cum_crisis_ind_bvx rzone_bus rzone_hh, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx rzone_bus rzone_hh rzone_both, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx rzone_both, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx rzone_either, fe lag(0)

* Crisis within 2 years
xtsccfixedb f2_cum_crisis_ind_bvx rzone_bus rzone_hh, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx rzone_bus rzone_hh rzone_both, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx rzone_both, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx rzone_either, fe lag(3)

* Crisis within 3 years
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus rzone_hh, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus rzone_hh rzone_both, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_both, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_either, fe lag(5)

* Crisis within 4 years
xtsccfixedb f4_cum_crisis_ind_bvx rzone_bus rzone_hh, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx rzone_bus rzone_hh rzone_both, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx rzone_both, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx rzone_either, fe lag(6)

log close

/******************************************************************************/
*
* Table 7
* Crisis Prediction with Global R-Zones
*
/******************************************************************************/

* Define global R-Zone variables
local varlist = "rzone_bus rzone_hh"
foreach var of local varlist {
	egen global_`var' = mean(`var'), by(year)
}

log using table7.log, replace

* Crisis within 1 year
xtsccfixedb f1_cum_crisis_ind_bvx rzone_bus global_rzone_bus if sample_bus == 1, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx rzone_hh global_rzone_hh if sample_hh == 1, fe lag(0)
xtsccfixedb f1_cum_crisis_ind_bvx rzone_bus global_rzone_bus rzone_hh global_rzone_hh if sample_hh == 1, fe lag(0)	

* Crisis within 2 years
xtsccfixedb f2_cum_crisis_ind_bvx rzone_bus global_rzone_bus if sample_bus == 1, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx rzone_hh global_rzone_hh if sample_hh == 1, fe lag(3)
xtsccfixedb f2_cum_crisis_ind_bvx rzone_bus global_rzone_bus rzone_hh global_rzone_hh if sample_hh == 1, fe lag(3)

* Crisis within 3 years
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus global_rzone_bus if sample_bus == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_hh global_rzone_hh if sample_hh == 1, fe lag(5)
xtsccfixedb f3_cum_crisis_ind_bvx rzone_bus global_rzone_bus rzone_hh global_rzone_hh if sample_hh == 1, fe lag(5)

* Crisis within 4 years
xtsccfixedb f4_cum_crisis_ind_bvx rzone_bus global_rzone_bus if sample_bus == 1, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx rzone_hh global_rzone_hh if sample_hh == 1, fe lag(6)
xtsccfixedb f4_cum_crisis_ind_bvx rzone_bus global_rzone_bus rzone_hh global_rzone_hh if sample_hh == 1, fe lag(6)

log close


/******************************************************************************/
*
* Table 8
* Probability of Experiencing severe economic decline by price and debt growth quantiles
*
/******************************************************************************/

* Define group indicators
forvalues p = 1/3 {
	forvalues d = 1/5 {
		gen bus_bin_`p'`d' = (price_bus_terc == `p')*(debt_bus_quint == `d')
		gen hh_bin_`p'`d'   = (price_hh_terc == `p')*(debt_hh_quint == `d')
	}
}

* Table 8a: Business Sector
log using table8a.log, replace

* Probability of Economic Contraction within 1-4 years by business debt growth quintile and equity price growth tercile
reg f1_cum_contract_gdp_ind bus_bin_* if sample_bus == 1, noconstant
reg f2_cum_contract_gdp_ind bus_bin_* if sample_bus == 1, noconstant
reg f3_cum_contract_gdp_ind bus_bin_* if sample_bus == 1, noconstant
reg f4_cum_contract_gdp_ind bus_bin_* if sample_bus == 1, noconstant

* Compare Probability of Economic Contraction to median group
drop bus_bin_23
xtsccfixedb f1_cum_contract_gdp_ind bus_bin_* if sample_bus == 1, lag(0)
xtsccfixedb f2_cum_contract_gdp_ind bus_bin_* if sample_bus == 1, lag(3)
xtsccfixedb f3_cum_contract_gdp_ind bus_bin_* if sample_bus == 1, lag(5)
xtsccfixedb f4_cum_contract_gdp_ind bus_bin_* if sample_bus == 1, lag(6)

log close

* Table 8b: Household *
log using table8b.log, replace

* Probability of Economic Contraction within 1-4 years by household debt growth quintile and house price growth tercile
reg f1_cum_contract_gdp_ind hh_bin_* if sample_hh == 1, noconstant
reg f2_cum_contract_gdp_ind hh_bin_* if sample_hh == 1, noconstant
reg f3_cum_contract_gdp_ind hh_bin_* if sample_hh == 1, noconstant
reg f4_cum_contract_gdp_ind hh_bin_* if sample_hh == 1, noconstant

* Compare Probability of Economic Contraction to median group
drop hh_bin_23
xtsccfixedb f1_cum_contract_gdp_ind hh_bin_* if sample_hh == 1, lag(0)
xtsccfixedb f2_cum_contract_gdp_ind hh_bin_* if sample_hh == 1, lag(3)
xtsccfixedb f3_cum_contract_gdp_ind hh_bin_* if sample_hh == 1, lag(5)
xtsccfixedb f4_cum_contract_gdp_ind hh_bin_* if sample_hh == 1, lag(6)

drop bus_bin_* hh_bin_*

log close

/******************************************************************************/
*
* Table 9
* Cumulative GDP growth by Price and Debt Growth Quantiles
*
/******************************************************************************/

* Define group indicators
forvalues p = 1/3 {
	forvalues d = 1/5 {
		gen bus_bin_`p'`d' = (price_bus_terc == `p')*(debt_bus_quint == `d')
		gen hh_bin_`p'`d'   = (price_hh_terc == `p')*(debt_hh_quint == `d')
	}
}


* Table 9a: Business Sector
log using table9a.log, replace

* 1-4 year cumulative GDP growth by business debt growth quintile and equity price growth tercile
reg f1_cum_gdp_real bus_bin_* if sample_bus == 1, noconstant
reg f2_cum_gdp_real bus_bin_* if sample_bus == 1, noconstant
reg f3_cum_gdp_real bus_bin_* if sample_bus == 1, noconstant
reg f4_cum_gdp_real bus_bin_* if sample_bus == 1, noconstant

* Compare Probability of Economic Contraction to median group
drop bus_bin_23
xtsccfixedb f1_cum_gdp_real bus_bin_* if sample_bus == 1, lag(0)
xtsccfixedb f2_cum_gdp_real bus_bin_* if sample_bus == 1, lag(3)
xtsccfixedb f3_cum_gdp_real bus_bin_* if sample_bus == 1, lag(5)
xtsccfixedb f4_cum_gdp_real bus_bin_* if sample_bus == 1, lag(6)

log close

* Table 9b: Household *
log using table9b.log, replace

* 1-4 year cumulative GDP growth by household debt growth quintile and house price growth tercile
reg f1_cum_gdp_real hh_bin_* if sample_hh == 1, noconstant
reg f2_cum_gdp_real hh_bin_* if sample_hh == 1, noconstant
reg f3_cum_gdp_real hh_bin_* if sample_hh == 1, noconstant
reg f4_cum_gdp_real hh_bin_* if sample_hh == 1, noconstant

* Compare Probability of Economic Contraction to median group
drop hh_bin_23
xtsccfixedb f1_cum_gdp_real hh_bin_* if sample_hh == 1, lag(0)
xtsccfixedb f2_cum_gdp_real hh_bin_* if sample_hh == 1, lag(3)
xtsccfixedb f3_cum_gdp_real hh_bin_* if sample_hh == 1, lag(5)
xtsccfixedb f4_cum_gdp_real hh_bin_* if sample_hh == 1, lag(6)

log close

drop bus_bin_* hh_bin_*

/******************************************************************************/
*
* Table 10
* Cumulative GDP growth by Price and Debt Growth Quantiles
*
/******************************************************************************/

drop *mod*

** First we need to fill in the R-Zone indicator variables for 2013-2015:

* Fill back rzone_bus for 2013-2015
gen debt_bus_quint_mod = debt_bus_quint
gsort -debt_to_gdp_bus_d3
replace debt_bus_quint_mod = debt_bus_quint_mod[_n-1] if debt_to_gdp_bus_d3 != . & debt_bus_quint_mod == . & debt_bus_quint_mod[_n-1] != .
sort debt_to_gdp_bus_d3
replace debt_bus_quint_mod = debt_bus_quint_mod[_n-1] if debt_to_gdp_bus_d3 != . & debt_bus_quint_mod == . & debt_bus_quint_mod[_n-1] != .	
gen price_bus_terc_mod = price_bus_terc		
gsort -equities_real_lg3
replace price_bus_terc_mod = price_bus_terc_mod[_n-1] if debt_to_gdp_bus_d3 != . & price_bus_terc_mod == . & price_bus_terc_mod[_n-1] != .
sort equities_real_lg3
replace price_bus_terc_mod = price_bus_terc_mod[_n-1] if debt_to_gdp_bus_d3 != . & price_bus_terc_mod == . & price_bus_terc_mod[_n-1] != .
gen rzone_bus_mod = (debt_bus_quint_mod>=5)*(price_bus_terc_mod>=3) if debt_bus_quint_mod != . & price_bus_terc_mod != . 


* Fill back rzone_hh for 2013-2015	
gen debt_hh_quint_mod = debt_hh_quint
gsort -debt_to_gdp_hh_d3
replace debt_hh_quint_mod = debt_hh_quint_mod[_n-1] if debt_to_gdp_hh_d3 != . & debt_hh_quint_mod == . & debt_hh_quint_mod[_n-1] != .
sort debt_to_gdp_hh_d3
replace debt_hh_quint_mod = debt_hh_quint_mod[_n-1] if debt_to_gdp_hh_d3 != . & debt_hh_quint_mod == . & debt_hh_quint_mod[_n-1] != .
gen price_hh_terc_mod = price_hh_terc		
gsort -housing_real_lg3
replace price_hh_terc_mod = price_hh_terc_mod[_n-1] if debt_to_gdp_hh_d3 != . & price_hh_terc_mod == . & price_hh_terc_mod[_n-1] != .
sort housing_real_lg3
replace price_hh_terc_mod = price_hh_terc_mod[_n-1] if debt_to_gdp_hh_d3 != . & price_hh_terc_mod == . & price_hh_terc_mod[_n-1] != .	
gen rzone_hh_mod = (debt_hh_quint_mod>=5)*(price_hh_terc_mod>=3) if debt_hh_quint_mod != . & price_hh_terc_mod != . 

* Define combined R-Zone indicator variables
gen rzone_both_mod = rzone_bus_mod*rzone_hh_mod
gen rzone_either_mod = max(rzone_bus_mod,rzone_hh_mod)

xtset

log using table10a.log, replace
** Calculate PPV, TPR and TNR for different R-Zone specifications

* Business R-Zone (PPV, TPR and TNR)
tab f3_cum_crisis_ind_bvx if rzone_bus == 1
gen prev_rzone_bus = max(l.rzone_bus_mod,l2.rzone_bus_mod,l3.rzone_bus_mod) if year < 2016 & l.rzone_bus_mod !=. & l2.rzone_bus_mod !=. & l3.rzone_bus_mod !=.
tab prev_rzone_bus if crisis_ind_bvx == 100
tab prev_rzone_bus if crisis_ind_bvx == 0

* Household R-Zone (PPV, TPR and TNR)
tab f3_cum_crisis_ind_bvx if rzone_hh == 1
gen prev_rzone_hh = max(l.rzone_hh_mod,l2.rzone_hh_mod,l3.rzone_hh_mod) if year < 2016 & l.rzone_hh_mod !=. & l2.rzone_hh_mod !=. & l3.rzone_hh_mod !=.
tab prev_rzone_hh if crisis_ind_bvx == 100
tab prev_rzone_hh if crisis_ind_bvx == 0	

* Either sector in R-Zone (PPV, TPR and TNR)
tab f3_cum_crisis_ind_bvx if rzone_either == 1
gen prev_rzone_either = max(l.rzone_either_mod,l2.rzone_either_mod,l3.rzone_either_mod) if year < 2016 & l.rzone_either_mod !=. & l2.rzone_either_mod !=. & l3.rzone_either_mod !=.
tab prev_rzone_either if crisis_ind_bvx == 100
tab prev_rzone_either if crisis_ind_bvx == 0

* Both sectors in R-Zone (PPV, TPR and TNR)
tab f3_cum_crisis_ind_bvx if rzone_both == 1
gen prev_rzone_both = max(l.rzone_both_mod,l2.rzone_both_mod,l3.rzone_both_mod) if year < 2016 & l.rzone_both_mod !=. & l2.rzone_both_mod !=. & l3.rzone_both_mod !=.
tab prev_rzone_both if crisis_ind_bvx == 100
tab prev_rzone_both if crisis_ind_bvx == 0
		
** Calculate how early the true positives are flagged by different R-Zones:
* Business R-Zone (years early)
gen rzone_bus_start = .
replace rzone_bus_start = year if rzone_bus == 1 & l.rzone_bus == 0
replace rzone_bus_start = year if rzone_bus == 1 & l.rzone_bus == .
replace rzone_bus_start = year if rzone_bus == 1 & l.rzone_bus == 0
replace rzone_bus_start = l.rzone_bus_start if rzone_bus == 1 & l.rzone_bus == 1
gen prev_rzone_bus_start = min(l.rzone_bus_start,l2.rzone_bus_start,l3.rzone_bus_start)
gen prev_rzone_bus_early = year - prev_rzone_bus_start
sum prev_rzone_bus_early if prev_rzone_bus == 1 & crisis_ind_bvx == 100

* Household R-Zone (years early) 
gen rzone_hh_start = .
replace rzone_hh_start = year if rzone_hh == 1 & l.rzone_hh == 0
replace rzone_hh_start = year if rzone_hh == 1 & l.rzone_hh == .
replace rzone_hh_start = year if rzone_hh == 1 & l.rzone_hh == 0
replace rzone_hh_start = l.rzone_hh_start if rzone_hh == 1 & l.rzone_hh == 1
gen prev_rzone_hh_start = min(l.rzone_hh_start,l2.rzone_hh_start,l3.rzone_hh_start)
gen prev_rzone_hh_early = year - prev_rzone_hh_start
sum prev_rzone_hh_early if prev_rzone_hh == 1 & crisis_ind_bvx == 100

* Either sector in R-Zone (years early)
gen rzone_either_start = .
replace rzone_either_start = year if rzone_either == 1 & l.rzone_either == 0
replace rzone_either_start = year if rzone_either == 1 & l.rzone_either == .
replace rzone_either_start = year if rzone_either == 1 & l.rzone_either == 0
replace rzone_either_start = l.rzone_either_start if rzone_either == 1 & l.rzone_either == 1
gen prev_rzone_either_start = min(l.rzone_either_start,l2.rzone_either_start,l3.rzone_either_start)
gen prev_rzone_either_early = year - prev_rzone_either_start
sum prev_rzone_either_early if prev_rzone_either == 1 & crisis_ind_bvx == 100

* Both sectors in R-Zone (years early)
gen rzone_both_start = .
replace rzone_both_start = year if rzone_both == 1 & l.rzone_both == 0
replace rzone_both_start = year if rzone_both == 1 & l.rzone_both == .
replace rzone_both_start = year if rzone_both == 1 & l.rzone_both == 0
replace rzone_both_start = l.rzone_both_start if rzone_both == 1 & l.rzone_both == 1
gen prev_rzone_both_start = min(l.rzone_both_start,l2.rzone_both_start,l3.rzone_both_start)
gen prev_rzone_both_early = year - prev_rzone_both_start
sum prev_rzone_both_early if prev_rzone_both == 1 & crisis_ind_bvx == 100	

log close




** Define Y-Zone variables: 
gen yzone_bus = (price_bus_terc >= 2)*(debt_bus_quint >= 4) if sample_bus == 1
gen yzone_hh = (price_hh_terc >= 2)*(debt_hh_quint >= 4) if sample_hh == 1
gen yzone_both = yzone_bus*yzone_hh
gen yzone_either = max(yzone_bus,yzone_hh)

gen yzone_bus_mod = (debt_bus_quint_mod>=4)*(price_bus_terc_mod>=2) if debt_bus_quint_mod != . & price_bus_terc_mod != . 
gen yzone_hh_mod = (debt_hh_quint_mod>=4)*(price_hh_terc_mod>=2) if debt_hh_quint_mod != . & price_hh_terc_mod != . 
gen yzone_both_mod = yzone_bus_mod*yzone_hh_mod
gen yzone_either_mod = max(yzone_bus_mod,yzone_hh_mod)

log using table10b.log, replace	
** Calculate PPV, TPR and TNR for different Y-Zone specifications
* Business Y-Zone (PPV, TPR and TNR)
tab f3_cum_crisis_ind_bvx if yzone_bus == 1
gen prev_yzone_bus = max(l.yzone_bus_mod,l2.yzone_bus_mod,l3.yzone_bus_mod) if year < 2016 & l.yzone_bus_mod !=. & l2.yzone_bus_mod !=. & l3.yzone_bus_mod !=.
tab prev_yzone_bus if crisis_ind_bvx == 100
tab prev_yzone_bus if crisis_ind_bvx == 0

* Household Y-Zone (PPV, TPR and TNR)
tab f3_cum_crisis_ind_bvx if yzone_hh == 1
gen prev_yzone_hh = max(l.yzone_hh_mod,l2.yzone_hh_mod,l3.yzone_hh_mod) if year < 2016 & l.yzone_hh_mod !=. & l2.yzone_hh_mod !=. & l3.yzone_hh_mod !=.
tab prev_yzone_hh if crisis_ind_bvx == 100
tab prev_yzone_hh if crisis_ind_bvx == 0	

* Either sector in Y-Zone (PPV, TPR and TNR)
tab f3_cum_crisis_ind_bvx if yzone_either == 1
gen prev_yzone_either = max(l.yzone_either_mod,l2.yzone_either_mod,l3.yzone_either_mod) if year < 2016 & l.yzone_either_mod !=. & l2.yzone_either_mod !=. & l3.yzone_either_mod !=.
tab prev_yzone_either if crisis_ind_bvx == 100
tab prev_yzone_either if crisis_ind_bvx == 0

* Both sectors in Y-Zone (PPV, TPR and TNR)
tab f3_cum_crisis_ind_bvx if yzone_both == 1
gen prev_yzone_both = max(l.yzone_both_mod,l2.yzone_both_mod,l3.yzone_both_mod) if year < 2016 & l.yzone_both_mod !=. & l2.yzone_both_mod !=. & l3.yzone_both_mod !=.
tab prev_yzone_both if crisis_ind_bvx == 100
tab prev_yzone_both if crisis_ind_bvx == 0

** Calculate how early the true positives are flagged by different Y-Zones:

* Business sector in Y-Zone (years early)
gen yzone_bus_start = .
replace yzone_bus_start = year if yzone_bus == 1 & l.yzone_bus == 0
replace yzone_bus_start = year if yzone_bus == 1 & l.yzone_bus == .
replace yzone_bus_start = year if yzone_bus == 1 & l.yzone_bus == 0
replace yzone_bus_start = l.yzone_bus_start if yzone_bus == 1 & l.yzone_bus == 1
gen prev_yzone_bus_start = min(l.yzone_bus_start,l2.yzone_bus_start,l3.yzone_bus_start)
gen prev_yzone_bus_early = year - prev_yzone_bus_start
sum prev_yzone_bus_early if prev_yzone_bus == 1 & crisis_ind_bvx == 100

* Household sector in Y-Zone (years early)
gen yzone_hh_start = .
replace yzone_hh_start = year if yzone_hh == 1 & l.yzone_hh == 0
replace yzone_hh_start = year if yzone_hh == 1 & l.yzone_hh == .
replace yzone_hh_start = year if yzone_hh == 1 & l.yzone_hh == 0
replace yzone_hh_start = l.yzone_hh_start if yzone_hh == 1 & l.yzone_hh == 1
gen prev_yzone_hh_start = min(l.yzone_hh_start,l2.yzone_hh_start,l3.yzone_hh_start)
gen prev_yzone_hh_early = year - prev_yzone_hh_start
sum prev_yzone_hh_early if prev_yzone_hh == 1 & crisis_ind_bvx == 100

* Either sector in Y-Zone (years early)
gen yzone_either_start = .
replace yzone_either_start = year if yzone_either == 1 & l.yzone_either == 0
replace yzone_either_start = year if yzone_either == 1 & l.yzone_either == .
replace yzone_either_start = year if yzone_either == 1 & l.yzone_either == 0
replace yzone_either_start = l.yzone_either_start if yzone_either == 1 & l.yzone_either == 1
gen prev_yzone_either_start = min(l.yzone_either_start,l2.yzone_either_start,l3.yzone_either_start)
gen prev_yzone_either_early = year - prev_yzone_either_start
sum prev_yzone_either_early if prev_yzone_either == 1 & crisis_ind_bvx == 100

* Both sectors in Y-Zone (years early)
gen yzone_both_start = .
replace yzone_both_start = year if yzone_both == 1 & l.yzone_both == 0
replace yzone_both_start = year if yzone_both == 1 & l.yzone_both == .
replace yzone_both_start = year if yzone_both == 1 & l.yzone_both == 0
replace yzone_both_start = l.yzone_both_start if yzone_both == 1 & l.yzone_both == 1
gen prev_yzone_both_start = min(l.yzone_both_start,l2.yzone_both_start,l3.yzone_both_start)
gen prev_yzone_both_early = year - prev_yzone_both_start
sum prev_yzone_both_early if prev_yzone_both == 1 & crisis_ind_bvx == 100	

log close

drop prev* yzone*



/******************************************************************************/
*
* Table 11
* GDP growth following True and False positives
*
/******************************************************************************/


* Define true and false positives
forvalues j = 1/4 {
	gen rzone_bus_tp`j' = rzone_bus*f`j'_cum_crisis_ind_bvx/100
	gen rzone_bus_fp`j' = rzone_bus*(1 - f`j'_cum_crisis_ind_bvx/100)	
	gen rzone_hh_tp`j'   = rzone_hh*f`j'_cum_crisis_ind_bvx/100
	gen rzone_hh_fp`j'   = rzone_hh*(1 - f`j'_cum_crisis_ind_bvx/100)
}

log using table11a.log, replace

** Business R-zone: Cumulative GDP growth following true and false positives
xtsccfixedb f1_cum_gdp_real rzone_bus_tp3 rzone_bus_fp3 if sample_bus == 1, fe lag(0)
xtsccfixedb f2_cum_gdp_real rzone_bus_tp3 rzone_bus_fp3 if sample_bus == 1, fe lag(3)
xtsccfixedb f3_cum_gdp_real rzone_bus_tp3 rzone_bus_fp3 if sample_bus == 1, fe lag(5)
xtsccfixedb f4_cum_gdp_real rzone_bus_tp3 rzone_bus_fp3 if sample_bus == 1, fe lag(6)
log close	

log using table11b.log, replace	
* Households R-zone: Cumulative GDP growth following true and false positives
xtsccfixedb f1_cum_gdp_real rzone_hh_tp3 rzone_hh_fp3 if sample_hh == 1, fe lag(0)
xtsccfixedb f2_cum_gdp_real rzone_hh_tp3 rzone_hh_fp3 if sample_hh == 1, fe lag(3)
xtsccfixedb f3_cum_gdp_real rzone_hh_tp3 rzone_hh_fp3 if sample_hh == 1, fe lag(5)
xtsccfixedb f4_cum_gdp_real rzone_hh_tp3 rzone_hh_fp3 if sample_hh == 1, fe lag(6)

log close	
